﻿using TravelCompanyDatabaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TravelCompanyContracts.BindingModels.TouristBindingModels;
using TravelCompanyContracts.SearchModels.TouristSearchModels;
using TravelCompanyContracts.StoragesContracts.TouristStoragesContracts;
using TravelCompanyContracts.ViewModels.TouristViewModels;

namespace TravelCompanyDatabaseImplement.Implements
{
    public class ExcursionStorage : IExcursionStorage
    {
        public List<ExcursionViewModel> GetFullList()
        {
            using var context = new TravelCompanyDatabase();
            return context.Excursions
            .Select(x => x.GetViewModel)
           .ToList();
        }
        public List<ExcursionViewModel> GetFilteredList(ExcursionSearchModel
       model)
        {
            if (string.IsNullOrEmpty(model.ExcursionName))
            {
                return new();
            }
            using var context = new TravelCompanyDatabase();
            return context.Excursions
            .Where(x => x.ExcursionName.Contains(model.ExcursionName))
           .Select(x => x.GetViewModel)
           .ToList();
        }
        public ExcursionViewModel? GetElement(ExcursionSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ExcursionName) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new TravelCompanyDatabase();
            return context.Excursions
            .FirstOrDefault(x =>
           (!string.IsNullOrEmpty(model.ExcursionName) && x.ExcursionName ==
           model.ExcursionName) ||
            (model.Id.HasValue && x.Id == model.Id))
            ?.GetViewModel;
        }
        public ExcursionViewModel? Insert(ExcursionBindingModel model)
        {
            var newExcursion = Excursion.Create(model);
            if (newExcursion == null)
            {
                return null;
            }
            using var context = new TravelCompanyDatabase();
            context.Excursions.Add(newExcursion);
            context.SaveChanges();
            return newExcursion.GetViewModel;
        }
        public ExcursionViewModel? Update(ExcursionBindingModel model)
        {
            using var context = new TravelCompanyDatabase();
            var component = context.Excursions.FirstOrDefault(x => x.Id ==
           model.Id);
            if (component == null)
            {
                return null;
            }
            component.Update(model);
            context.SaveChanges();
            return component.GetViewModel;
        }
        public ExcursionViewModel? Delete(ExcursionBindingModel model)
        {
            using var context = new TravelCompanyDatabase();
            var element = context.Excursions.FirstOrDefault(rec => rec.Id ==
           model.Id);
            if (element != null)
            {
                context.Excursions.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
